{ This unit overrides TEnumproperty and also TColor }
{ To present more english versions of their text }
unit enumped;

interface

uses Dsgnintf, Typinfo, classes, graphics, sysutils;

type
{ TEnumProperty
  The default property editor for all enumerated properties (e.g. TShape =
  (sCircle, sTriangle, sSquare), etc.). }

  TNiceEnumProperty = class(TEnumProperty)
  Protected
    FCurrentProc : TGetStrProc;
    FGetValuesProc : TList;
    FTranslate : TStringList;
    Procedure AddGetValuesProc(proc : TGetStrProc);
    Procedure RemoveGetValuesProc(proc : TGetStrProc);
    procedure FilterGotValues(const Value : string);
    procedure AddTranslation(const Value: string);
    Function Translate(const Code : String) : String;
    Procedure PrepareTranslation;
  public
    function GetValue: string; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
    Destructor Destroy; override;
  end;

implementation

type
  TDummy=(a,b,c);

Function NiceValue(PropertyEditor : TPropertyEditor; Value : String) : String;
  Function Nicely(S : String) : String;
  var
    FoundUpper : Boolean;
    C : integer;
  begin
    Result:='';
    FoundUpper:=False;
    c:=1; while c<length(s) do begin
      if (s[c]>='A') and (s[c]<='Z') then begin
        Result:=Copy(s,c,length(s)-c+1);
        FoundUpper:=True;
        Break;
      end;
      inc(c);
    end;
    If not FoundUpper then Result:=S;
  end;

var
  TI : PTypeInfo;
begin
  { for enumerated types make it look nice }
  { also check by hand for registered integer types, TColor, TCursor, TShortCut }
  { we can't get a list of registered integers so we check by hand for known types }
  If PropertyEditor is TColorProperty then begin
    Result:=Nicely(Value);
  end else begin
    TI:=PropertyEditor.GetProptype;
    Case TI^.Kind of
      tkEnumeration: begin
        Result:=Nicely(Value);
      end;
    else
      Result:=Value;
    end;
  end;
end;

Function TNiceEnumProperty.GetValue;
begin
  Result:=NiceValue(Self,Inherited GetValue);
end;

Procedure TNiceEnumProperty.GetValues;
begin
  AddGetValuesProc(Proc);
  try
    Inherited GetValues(FilterGotValues);
  finally
    RemoveGetValuesProc(Proc);
  end;
end;

Procedure TNiceEnumProperty.SetValue;
begin
  Inherited SetValue(Translate(Value));
end;

Function TNiceEnumProperty.Translate;
begin
  If not Assigned(FTranslate) then PrepareTranslation;
  Result:=FTranslate.Values[Code];
  If Result='' then Result:=Code;
end;

Procedure TNiceEnumProperty.PrepareTranslation;
begin
  If not Assigned(FTranslate) then begin
    FTranslate:=TStringList.Create;
    Inherited GetValues(AddTranslation);
  end;
end;

Procedure TNiceEnumProperty.AddTranslation;
begin
  FTranslate.Add(Value+'='+NiceValue(Self,Value));
end;

Procedure TNiceEnumproperty.AddGetValuesProc;
begin
  If not Assigned(FGetValuesProc) then FGetValuesProc:=TList.Create;
  FGetValuesProc.Add(TMethod(Proc).Data);
  FGetValuesProc.Add(TMethod(Proc).Code);
  FCurrentProc:=Proc;
end;

Procedure TNiceEnumProperty.RemoveGetValuesProc;
var
  C : Integer;
begin
  C:=FGetValuesProc.Count;
  FGetValuesProc.Delete(C-1);
  FGetValuesProc.Delete(C-2);
  FGetValuesProc.Pack;
  C:=FGetValuesProc.Count;
  If C>0 then begin
    TMethod(FCurrentProc).Code:=FGetValuesProc[c-1];
    TMethod(FCurrentProc).Code:=FGetValuesProc[c-1];
  end;
end;

Procedure TNiceEnumProperty.FilterGotValues;
begin
  FCurrentProc(Value);
end;

Destructor TNiceEnumProperty.Destroy;
begin
  FTranslate.Free;
  FGetValuesProc.Free;
end;

begin
  RegisterPropertyEditor(TypeInfo(TDummy),nil,'',TNiceEnumProperty);
end.

