unit Handles;
{ this form defines control handles for designer }
{ They are not delphi controls cos its the easiest way to avoid 'AlignControl' wars }
{ where the handles keep getting in the way with their visble and position changes }
{ and the scrollby changes which happen TO them }

interface

uses wintypes, winprocs, classes, controls, messages, sysutils, forms;

const
  WName='TXControlhandle';
  { Handle dimensions }
  cHandleWidth=7;
  cHandleHeight=chandleWidth;

  DefaultWindowClass: TWndClass = (
    style: 0;
    lpfnWndProc: nil;
    cbClsExtra: 0;
    cbWndExtra: 6;
    hInstance: 0;
    hIcon: 0;
    hCursor: 0;
    hbrBackground: 0;
    lpszMenuName: nil;
    lpszClassName: nil);

  SizeCursors : Array[0..7] of TCursor=
    (crSizeNS, crSizeNESW, crSizeWE, crSizeNWSE, crSizeNS, crSizeNESW, crSizeWE, crSizeNWSE);

Type
  THandleType=(thtN,thtNE,thtE,thtSE,thtS,thtSW,thtW,thtNW);

  TXControlHandle=class(TComponent)
  protected
    FHandle : HWND;
    FParentHandle : HWND;
    FhandleType : THandleType;
    FVisible : Boolean;
    FCursor : TCursor;
    Procedure Start(Parent : HWND);
  public
    Constructor Create(AOwner : TComponent; HandleType : THandleType);
    Procedure SetParent(Handle : HWND);
    Procedure SetState(Handle : HWND; Visible : Boolean; Left, Top, Width, Height : Integer);
    Procedure SetVisible(Handle : HWND; Visible : Boolean);
    Procedure SetOnTop;
  end;

implementation

uses fedit2;

const
  CursorIds : Array [0..7] of pointer=(IDC_SizeNS, IDC_SizeNESW, IDC_SizeWE, IDC_SizeNWSE,
                                       IDC_SizeNS, IDC_SizeNESW, IDC_SizeWE, IDC_SizeNWSE);
  Made : Array[0..7] of boolean=(false, false, false, false, false, false, false, false);

function WindowProc(Window: HWnd; Message, WParam: Word;
  LParam: Longint): Longint; export;

  Procedure WMLButtonDown;
  var
    Form : TForm;
    Designer : TFormEditor2;
    Self : TXControlHandle;
  begin
    Self:=TXControlHandle(GetWindowLong(Window,2));
    Form:=FindControl(GetParent(Window)) as TForm;
    If (csDesigning in Form.ComponentState) and (Form.Designer<>nil) then begin
      Designer:=(Form.Designer as TFormEditor2);
      ClientToScreen(Window,TPoint(LParam));
      Designer.PointInScreen:=TPoint(LParam);
      Designer.PointInForm:=Form.ScreenToClient(Designer.PointInScreen);
      Designer.SizingHandle:=Self.FHandleType;
      Designer.SizeMessage;
    end;
  end;

  Procedure WMLButtonUp;
  { This will rarely happen, only when component gets mouse down where we appear and so we get mouse up }
  { just pass on to IsDesignMessage }
  var
    Form : TForm;
    Designer : TFormEditor2;
    Self : TXControlHandle;
    MessageArray : TMessage;
  begin
    Self:=TXControlHandle(GetWindowLong(Window,2));
    Form:=FindControl(GetParent(Window)) as TForm;
    If (csDesigning in Form.ComponentState) and (Form.Designer<>nil) then begin
      Designer:=(Form.Designer as TFormEditor2);
      MessageArray.Msg:=Message;
      MessageArray.WParam:=WParam;
      MessageArray.LParam:=LParam;
      Designer.IsDesignMsg(Designer.Form{has to be something},MessageArray);
    end;
  end;

  Procedure WMPaint;
  Var
    DC : THandle;
    PS : TPaintStruct;
  begin
    If WParam=0 then
      DC:=BeginPaint(Window,PS)
    else
      DC:=WParam;
    PatBlt(DC,0,0,cHandleWidth,cHandleHeight,Blackness);
    If WParam=0 then
      EndPaint(Window,PS);
    Result:=0;
  end;

  Procedure WMDestroy;
  var
    Self : TXControlHandle;
  begin
    Self:=TXControlHandle(GetWindowLong(Window,2));
    { Make a note }
    Self.FHandle:=0;
  end;

  Procedure WMEraseBKGND;
  begin
    Result:=1;
  end;

  Procedure MouseMessage;
  begin

  end;

begin
  Case Message of
    WM_Paint: WMPaint;
    WM_LButtonDown: WMLButtonDown;
    WM_LButtonUp: WMLButtonUp;
    WM_Destroy: WMDestroy;
    WM_EraseBKGND: WMEraseBKGND;
  else
    WindowProc := DefWindowProc(Window, Message, WParam, LParam);
  end;
end;

{------}

Constructor TXControlhandle.Create;
begin
  Inherited Create(AOwner);
  FHandleType:=HandleType;
end;

Procedure TXControlHandle.Start;
var
  WindowClass : TWndClass;
  C : Integer;
  CursorId : Pointer;
  Buffer : Array[0..255] of char;
  Classname : String;
begin
  FParentHandle:=Parent;
  CursorId:=nil;
  For c:=0 to SizeOf(SizeCursors) do
    If SizeCursors[c]=FCursor then begin
      CursorId:=CursorIds[c];
      Break;
    end;
  Classname:=WName+IntToStr(C);
  if (HPrevInst = 0) and not made[c] then
  begin
    WindowClass:=DefaultWindowClass;
    WindowClass.hInstance := HInstance;
    WindowClass.hCursor := LoadCursor(0, CursorId);
    WindowClass.hbrBackground := GetStockObject(white_Brush);
    WindowClass.lpfnWndProc:=@WindowProc;
    WindowClass.lpszClassName:=StrPCopy(Buffer,Classname);
    if not winprocs.RegisterClass(WindowClass) then
      Halt(255);
    Made[C]:=True;
  end;
  FHandle:=CreateWindowEX(
    0,
    StrPCopy(Buffer,Classname),
    '',
    ws_child or ws_clipsiblings,
    0,
    0,
    cHandleWidth,
    cHandleHeight,
    FParentHandle,
    0,
    HInstance,
    nil);
  SetWindowWord(FHandle,0,word(FHandleType));
  SetWindowLong(FHandle,2,longint(Self));
  UpdateWindow(FHandle);
end;

Procedure TXControlhandle.SetParent;
begin
  FCursor:=SizeCursors[Word(FHandleType)];
  Start(Handle);
end;

Procedure TXControlHandle.SetState;
var
  Rect : TRect;
begin
  If FHandle=0 then begin
    Start(Handle);
  end;
  {}
  If FHandle<>0 then begin
    GetWindowRect(FHandle,Rect);
    If (Rect.Top<>Top) or (Rect.Left<>Left) or
       ((Rect.Bottom-Rect.Top)<>Height) or ((Rect.Right-Rect.Left)<>Width) then
      SetWindowPos(FHandle, HWND_TOP, Left, Top, Width, Height,SWP_NOACTIVATE);
  end;
  SetVisible(Handle, Visible);
end;

Procedure TXControlHandle.SetVisible;
var
  PHandle : THandle;
  Rect : TRect;
begin
  If FHandle=0 then begin
    Start(Handle);
  end;

  If FHandle<>0 then begin
    If Visible<>FVisible then begin
      if Visible then
        SetWindowPos(FHandle, HWND_TOP, 0,0,0,0, SWP_NOACTIVATE or SWP_NOMOVE or SWP_NOSIZE or SWP_ShowWindow)
      else begin
        SetWindowPos(FHandle, HWND_TOP, 0,0,0,0, SWP_NOACTIVATE or SWP_NOMOVE or SWP_NOSIZE or SWP_HideWindow or SWP_NoRedraw);
        { Now invalidate region by hande }{ If we don't do SWP_NoRedraw then it all happens NOW }
        PHandle:=GetParent(FHandle);
        GetWindowRect(FHandle,Rect);
        ScreenToClient(PHandle,Rect.TopLeft);
        ScreenToClient(PHandle,Rect.BottomRight);
        RedrawWindow(PHandle,@Rect,0,RDW_Invalidate or RDW_Erase or RDW_Frame or RDW_AllChildren);
      end;
      FVisible:=Visible;
    end;
  end;
end;

Procedure TXControlHandle.SetOnTop;
begin
  If Fhandle<>0 then begin
    SetWindowPos(FHandle, HWND_TOP, 0,0,0,0, SWP_NOACTIVATE or SWP_NOMOVE or SWP_NOSIZE);
    UpdateWindow(FHandle);
  end;
end;

end.


